package com.muyu.etl.datasource;

import com.alibaba.druid.pool.DruidDataSource;
import com.muyu.common.log.annotation.Log;
import com.muyu.etl.domain.DataSource;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Component;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

/**
 * @ClassName MyDataSource
 * @Description 数据库连接池管理
 * @Author Xin.Yao
 * @Date 2024/5/9 19:36
 */
@Component
@Log4j2
public class MyDataSource {
    private static HashMap<String, DruidDataSource> pools = new HashMap<>();

    /**
     * 总数据库连接池初始化
     * @param dataSourceList
     */
    public static void info(List<DataSource> dataSourceList){
            dataSourceList.stream().forEach(dataSource -> {
                String jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
                String key = dataSource.getDataSourceName()+"_"+dataSource.getId();
                // 创建德鲁伊数据源
                DruidDataSource druidSource = new DruidDataSource();
                // 配置数据库连接信息
                druidSource.setUrl(jdbcUrl);
                druidSource.setUsername(dataSource.getUsername());
                druidSource.setPassword(dataSource.getPassword());
                // 配置连接池大小
                druidSource.setInitialSize(Integer.valueOf(dataSource.getInitNum().toString()));
                druidSource.setMinIdle(5);
                druidSource.setMaxActive(Integer.valueOf(dataSource.getMaxNum().toString()));
                try {
                    druidSource.init();
                } catch (SQLException e) {
                    throw new RuntimeException(e);
                }
                addDataSource(key,druidSource);
            });

    }

    /**
     * 添加新的连接池
     * @param key
     * @param druidDataSource
     */
    public static void addDataSource(String key,DruidDataSource druidDataSource){
        //判断pools中是否存在此数据接入的连接池
        if (pools.containsKey(key)){
            //如果存在就关闭并清除
            DruidDataSource druidDataSource1 = pools.get(key);
            druidDataSource1.close();
            pools.remove(key);
        }
        //添加新的连接池
        pools.put(key,druidDataSource);
    }

    /**
     * 根据 key 获取连接池中的连接
     * @param key
     * @return Connection
     */
    public static Connection getConnection(String key){
        try {
            //根据key获取连接池
            DruidDataSource druidDataSource = pools.get(key);
            // 从连接池中获取第一个链接
            return druidDataSource.getConnection();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 归还连接池中的连接
     * @param connection
     */

    public static void addBack(Connection connection){
        try {
            connection.close();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 根据 key 查看连接池连接信息
     * @param key
     */
    public static void size(String key){
        for (String s : pools.keySet()) {
            log.info("连接池名称: "+s);
        }
        //根据key获取连接池
        DruidDataSource druidDataSource = pools.get(key);
        //获取正在使用的连接数量
        int activeCount = druidDataSource.getActiveCount();
        //获取线程池中的连接数量
        int poolingCount = druidDataSource.getPoolingCount();
        log.info(key+" 连接池中正在使用连接: "+activeCount+"个 ,线程池中线程数量: "+poolingCount+"个");
    }
}
