package com.muyu.etl.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.etl.domain.AssetModel;
import com.muyu.etl.domain.DataAsset;
import com.muyu.etl.domain.DataSource;
import com.muyu.etl.service.IDataSourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 数据接入Controller
 *
 * @author ruoyi
 * @date 2024-04-20
 */
@RestController
@RequestMapping("/source")
public class DataSourceController extends BaseController
{
    @Autowired
    private IDataSourceService dataSourceService;

    /**
     * 查询数据接入列表
     */
    @PostMapping("/list")
    public Result<TableDataInfo<DataSource>> list(@RequestBody DataSource dataSource)
    {
        startPage();
        List<DataSource> list = dataSourceService.selectDataSourceList(dataSource);
        return getDataTable(list);
    }


    //获取数据源中表的数据(KVT结构)
    @PostMapping("/AssetsList")
    public Result assetsList(@RequestBody DataSource dataSource)
    {
        return dataSourceService.assetsList(dataSource);
    }


    //数据接入同步资产结构
    @PostMapping("/SynchronousData")
    public Result synchronousData(@RequestBody DataSource dataSource)
    {
        return dataSourceService.synchronousData(dataSource);
    }

    //根据数据接入信息获取资产表列表信息
    @PostMapping("/DataAssetList")
    public Result dataAssetList(@RequestBody DataSource dataSource)
    {
        return dataSourceService.dataAssetList(dataSource);
    }

    //统计选中数据接入的数据(数据接入量 数据接入表中字段量 数据接入表中数据量)
    @GetMapping("/Statistics")
    public Result statistics(){
        return dataSourceService.statistics();
    }

    //根据数据表信息获取资产模型字段列表信息
    @PostMapping("/AssetModelList")
    public Result assetModelList(@RequestBody DataAsset dataAsset)
    {
        return dataSourceService.assetModelList(dataAsset);
    }

    @PostMapping("/StructureList")
    public Result structureList(@RequestBody DataAsset dataAsset)
    {
        return dataSourceService.structureList(dataAsset);
    }

    /**
     * 导出【请填写功能名称】列表
     */
    @RequiresPermissions("system:source:export")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DataSource dataSource)
    {
        List<DataSource> list = dataSourceService.selectDataSourceList(dataSource);
        ExcelUtil<DataSource> util = new ExcelUtil<DataSource>(DataSource.class);
        util.exportExcel(response, list, "【请填写功能名称】数据");
    }

    /**
     * 获取【请填写功能名称】详细信息
     */
    @RequiresPermissions("system:source:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id)
    {
        return Result.success(dataSourceService.selectDataSourceById(id));
    }

    /**
     * 新增【请填写功能名称】
     */
    @RequiresPermissions("system:source:add")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody DataSource dataSource)
    {
        return toAjax(dataSourceService.insertDataSource(dataSource));
    }

    /**
     * 测试连接
     */
    @Log(title = "测试连接", businessType = BusinessType.INSERT)
    @PostMapping("/TestConnection")
    public Result testConnection(@RequestBody DataSource dataSource)
    {
        return dataSourceService.testConnection(dataSource);
    }

    /**
     * 修改【请填写功能名称】
     */
    @RequiresPermissions("system:source:edit")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody DataSource dataSource)
    {
        return toAjax(dataSourceService.updateDataSource(dataSource));
    }

    /**
     * 删除【请填写功能名称】
     */
    @RequiresPermissions("system:source:remove")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids)
    {
        return toAjax(dataSourceService.deleteDataSourceByIds(ids));
    }
}
