package com.muyu.etl.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.etl.domain.SourceAccredit;
import com.muyu.etl.domain.req.SourceAccreditReq;
import com.muyu.etl.service.ISourceAccreditService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 【请填写功能名称】Controller
 *
 * @author ruoyi
 * @date 2024-04-25
 */
@RestController
@RequestMapping("/sourceAccredit")
public class SourceAccreditController extends BaseController
{
    @Autowired
    private ISourceAccreditService sourceAccreditService;

    /**
     * 查询【请填写功能名称】列表
     */
    @RequiresPermissions("system:accredit:list")
    @GetMapping("/list")
    public Result<TableDataInfo<SourceAccredit>> list(SourceAccredit sourceAccredit)
    {
        startPage();
        List<SourceAccredit> list = sourceAccreditService.selectSourceAccreditList(sourceAccredit);
        return getDataTable(list);
    }

    /**
     * 导出【请填写功能名称】列表
     */
    @RequiresPermissions("system:accredit:export")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SourceAccredit sourceAccredit)
    {
        List<SourceAccredit> list = sourceAccreditService.selectSourceAccreditList(sourceAccredit);
        ExcelUtil<SourceAccredit> util = new ExcelUtil<SourceAccredit>(SourceAccredit.class);
        util.exportExcel(response, list, "【请填写功能名称】数据");
    }

    /**
     * 获取【请填写功能名称】详细信息
     */
    @RequiresPermissions("system:accredit:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id)
    {
        return success(sourceAccreditService.selectSourceAccreditById(id));
    }

    /**
     * 根据数据源id获取授权信息
     */
    @GetMapping(value = "/GetSourceAccreditByDataSourceId")
    public Result getSourceAccreditByDataSourceId(@RequestParam("id") Long id)
    {
        return sourceAccreditService.getSourceAccreditByDataSourceId(id);
    }

    /**
     * 新增【请填写功能名称】
     */
    @RequiresPermissions("system:accredit:add")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody SourceAccreditReq sourceAccreditReq)
    {
        return sourceAccreditService.insertSourceAccredit(sourceAccreditReq);
    }

    /**
     * 授权删除
     */
    @Log(title = "授权删除", businessType = BusinessType.INSERT)
    @PostMapping("/DeleteSourceAccreditBySourceIds")
    public Result deleteSourceAccreditBySourceIds(@RequestBody SourceAccreditReq sourceAccreditReq)
    {
        return sourceAccreditService.deleteSourceAccreditBySourceIds(sourceAccreditReq);
    }

    /**
     * 修改【请填写功能名称】
     */
    @RequiresPermissions("system:accredit:edit")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody SourceAccredit sourceAccredit)
    {
        return toAjax(sourceAccreditService.updateSourceAccredit(sourceAccredit));
    }

    /**
     * 删除【请填写功能名称】
     */
    @RequiresPermissions("system:accredit:remove")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids)
    {
        return toAjax(sourceAccreditService.deleteSourceAccreditByIds(ids));
    }
}
