package com.muyu.etl.service.impl;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.LoginUser;
import com.muyu.common.system.domain.SysRole;
import com.muyu.etl.domain.Dictionary;
import com.muyu.etl.domain.*;
import com.muyu.etl.domain.custom.AssetsModule;
import com.muyu.etl.domain.custom.Statistics;
import com.muyu.etl.domain.custom.VTClass;
import com.muyu.etl.feign.SysUserFeignService;
import com.muyu.etl.mapper.*;
import com.muyu.etl.service.IDataSourceService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.*;
import java.util.Date;
import java.util.*;
import java.util.concurrent.*;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-20
 */
@Service
@Log4j2
public class DataSourceServiceImpl implements IDataSourceService
{
    @Autowired
    private DataSourceMapper dataSourceMapper;

    @Autowired
    private DataAssetMapper dataAssetMapper;

    @Autowired
    private AssetModelMapper assetModelMapper;

    @Autowired
    private DictionaryDataMapper dictionaryDataMapper;

    @Autowired
    private DictionaryServiceImpl dictionaryService;

    @Autowired
    private SourceAccreditMapper sourceAccreditMapper;

    @Autowired
    private AssetAccreditMapper assetAccreditMapper;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    private final ExecutorService executorService = Executors.newFixedThreadPool(6);

    /**
     * 根据id查询数据接入集合
     *
     * @param id 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public DataSource selectDataSourceById(Long id)
    {
        DataSource dataSource = dataSourceMapper.selectDataSourceById(id);
        return dataSource;
    }

    /**
     * 查询数据接入列表
     *
     * @param dataSource 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<DataSource> selectDataSourceList(DataSource dataSource)
    {
        List<DataSource> dataSourceList = new ArrayList<DataSource>();
        List<DataSource> dataSources = dataSourceMapper.selectDataSourceList(dataSource);
        LoginUser loginUser = SecurityUtils.getLoginUser();
        List<SysRole> roles = new ArrayList<>();
        if (loginUser != null){
            roles = loginUser.getSysUser().getRoles();
        }else{
            roles.add(new SysRole(1L));
        }
        //判断登录人是否为管理员,不是则需要过滤掉未授权的信息
        if (roles.get(0).getRoleId()==1){
            dataSourceList = dataSources;
        }else{
            //数据接入过滤完的主键id集合
            ArrayList<Long> longs1 = new ArrayList<>();
            //已授权的数据接入id集合
            List<Long> sourceIds = sourceAccreditMapper.getSourceAccreditByUserId(SecurityUtils.getUserId()).stream().map(SourceAccredit::getDataSourceId).filter(Objects::nonNull).toList();
            //已授权数据模型表的主键id集合
            List<Long> assetIds = assetAccreditMapper.getAssetAccreditByUserId(SecurityUtils.getUserId()).stream().map(AssetAccredit::getDataAssetId).filter(Objects::nonNull).toList();
            //判断已授权数据模型表的主键id集合是否为空
            if (!assetIds.isEmpty()){
                //获取已授权的数据模型表的数据源id集合
                List<Long> longs = dataAssetMapper.getDataAssetByAssetId(assetIds).stream().map(DataAsset::getDataSourceId).toList();
                //添加到已授权的数据源id集合中
                longs1.addAll(longs);
            }
            longs1.addAll(sourceIds);
            //从所有的数据源信息集合中过滤掉未授权的数据源信息
            dataSourceList = dataSources.stream().filter(dataSourceInfo -> longs1.contains(dataSourceInfo.getId())).toList();
        }
        return dataSourceList;
    }

    /**
     * 新增数据接入信息
     *
     * @param dataSource 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int insertDataSource(DataSource dataSource)
    {
        dataSource.setCreateTime(DateUtils.getNowDate());
        dataSource.setCreateBy(SecurityUtils.getUsername());
        //判断添加的数据源数据库类型
        if ("MySql".equals(dataSource.getType())){
            dataSource.setJdbcDriver("com.mysql.cj.jdbc.Driver");
        }else{
            dataSource.setJdbcDriver("org.postgresql.Driver");
        }
        return dataSourceMapper.insertDataSource(dataSource);
    }

    /**
     * 修改数据接入信息
     *
     * @param dataSource 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int updateDataSource(DataSource dataSource)
    {
        dataSource.setUpdateTime(DateUtils.getNowDate());
        dataSource.setUpdateBy(SecurityUtils.getUsername());
        //判断修改的数据源数据库类型
        if ("MySql".equals(dataSource.getType())){
            dataSource.setJdbcDriver("com.mysql.cj.jdbc.Driver");
        }else{
            dataSource.setJdbcDriver("org.postgresql.Driver");
        }
        return dataSourceMapper.updateDataSource(dataSource);
    }

    /**
     * 批量删除数据接入
     *
     * @param ids 需要删除的【请填写功能名称】主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteDataSourceByIds(Long[] ids)
    {
        //删除数据接入的数据模型表及关联数据
        deleteChildLevel(ids,"delete");
        return dataSourceMapper.deleteDataSourceByIds(ids);
    }

    //删除数据接入的用户授权数据
    public void deleteAccredit(List<Long> sourceIds,List<Long> assetIds){
        //删除数据接入的用户授权数据
        if (!sourceIds.isEmpty()){
            sourceAccreditMapper.deleteSourceAccreditBySourceIds(sourceIds);
        }
        //删除数据接入的数据模型表及关联数据
        if (!assetIds.isEmpty()){
            assetAccreditMapper.deleteAssetAccreditByAssetIds(assetIds);
        }
    }

    //删除数据接入相关信息(数据表,数据表中字段,关联字典)
    public void deleteChildLevel(Long[] ids,String type){
        //获取数据模型表
        List<DataAsset> dataAssetList=dataAssetMapper.getDataAssetList(ids);
        //数据模型表主键id集合
        List<Long> dataAssetIds = new ArrayList<>();
        if (!dataAssetList.isEmpty()){
            //获取数据模型表主键id集合
            dataAssetIds = dataAssetList.stream().map(DataAsset::getId).toList();
            //获取数据模型表关联的字典表
            List<AssetModel> assetModelList=assetModelMapper.getAssetModelList(dataAssetIds);
            //根据数据模型表id删除数据模型表信息
            dataAssetMapper.deleteDataAssetByIds(dataAssetIds.toArray(Long[]::new));
            //根据数据模型表id删除数据模型表的字段信息
            if (!assetModelList.isEmpty()){
                List<Long> assetModelIds = assetModelList.stream().map(AssetModel::getId).toList();
                assetModelMapper.deleteAssetModelByIds(assetModelIds.toArray(Long[]::new));
            }
        }
        //删除数据接入的用户授权数据
        if ("delete".equals(type)){
            deleteAccredit(Arrays.stream(ids).toList(),dataAssetIds);
        }else{
            deleteAccredit(new ArrayList<Long>(),dataAssetIds);
        }

    }

    /**
     * 删除数据接入信息
     *
     * @param id 【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteDataSourceById(Long id)
    {
        return dataSourceMapper.deleteDataSourceById(id);
    }

    //测试连接

    @Override
    public Result testConnection(DataSource dataSource) {
        String jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
        if (dataSource.getConnectionParam()!=null && dataSource.getConnectionParam()!=""){
            jdbcUrl = jdbcUrl+"?"+dataSource.getConnectionParam();
        }
        Connection conn = null;
        try {
            Class.forName(dataSource.getJdbcDriver());
        } catch (ClassNotFoundException e) {
            return Result.error("连接失败");
        }
        try {
            conn = DriverManager.getConnection(jdbcUrl, dataSource.getUsername(), dataSource.getPassword());
            conn.close();
        }catch (Exception e){
            return Result.error("连接失败");
        }

        return Result.success("连接成功");
    }

    /**
     * 根据给定的数据源信息，建立数据库连接，并执行SQL查询。将查询结果转换为包含列名、数据类型与实际数据的列表。
     *
     * @return AssetsModule 包含查询结果和列信息的模块化对象。
     */
    //获取数据模型的数据(KVT结构)
    public AssetsModule getStructure(DataAsset dataAsset){
        DataSource dataSource = dataSourceMapper.selectDataSourceById(dataAsset.getDataSourceId());
        String jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
        if (dataSource.getConnectionParam()!=null && dataSource.getConnectionParam()!=""){
            jdbcUrl = jdbcUrl+"?"+dataSource.getConnectionParam();
        }
        Connection conn = null;
        List<Map<String, VTClass>> kvtList = new ArrayList<>();
        HashMap<String, String> map = new HashMap<>();
        try {
            // 加载 JDBC 驱动
            Class.forName(dataSource.getJdbcDriver());
            // 建立数据库连接
            conn = DriverManager.getConnection(jdbcUrl, dataSource.getUsername(), dataSource.getPassword());
        } catch (SQLException | ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        try {
            // 准备 SQL 语句并执行查询
            PreparedStatement pst = conn.prepareStatement(dataSource.getSql());
            ResultSet resultSet = pst.executeQuery();
            ResultSetMetaData rsd = resultSet.getMetaData();

            // 提取查询结果的列信息
//            for(int i = 1; i <= rsd.getColumnCount(); i++) {
//                String[] split = rsd.getColumnClassName(i).split("\\.");
//                map.put(rsd.getColumnName(i),split[split.length-1]);
//            }

            int columnCount = rsd.getColumnCount();
            // 遍历查询结果，将每行数据转换为 Map 形式存储
            while (resultSet.next()){
                Map<String, VTClass> stringVTClassHashMap = new HashMap<>();
                for (int i = 1; i <= columnCount; i++) {
                    // 获取列名和类型，并封装数据
                    String columnName = rsd.getColumnName(i);
                    String[] split = rsd.getColumnClassName(i).split("\\.");
                    String type = split[split.length-1];
                    Object value = resultSet.getObject(i);
                    if (value==null){
                        stringVTClassHashMap.put(columnName,new VTClass("",type));
                    }else{
                        stringVTClassHashMap.put(columnName,new VTClass(value.toString(),type));
                    }
                }
                kvtList.add(stringVTClassHashMap);
            }
            // 关闭资源
            pst.close();
            conn.close();
            pst = null;
            conn = null;
        } catch(SQLException e) {
            e.printStackTrace();
        }
        // 构建并返回 AssetsModule 对象
        return new AssetsModule().assetsModuleBuilder(kvtList,map);
    }

    //获取数据模型表内字段信息的映射类型map集合
    public Map<String,String> getTypeMap(DataSource dataSource, String tableName){
        String jdbcUrl="";
        String sql="";
        jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
        if (dataSource.getType().equals("MySql")){
            sql = "select * from "+tableName+" where 2=1";
        }else{
            sql = "select * from "+dataSource.getModeName()+"."+tableName+" where 2=1";
        }
        Connection conn = null;
        Map<String, String> map = new HashMap<>();
        try {
            conn = DriverManager.getConnection(jdbcUrl, dataSource.getUsername(), dataSource.getPassword());
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        try {
            PreparedStatement pst = null;
            pst = conn.prepareStatement(sql);
            ResultSet resultSet = pst.executeQuery();
            ResultSetMetaData rsd = resultSet.getMetaData();
            for(int i = 1; i <= rsd.getColumnCount(); i++) {
                String substring="";
                if (rsd.getColumnClassName(i).contains(".")){
                    String[] split = rsd.getColumnClassName(i).split("\\.");
                    substring = split[split.length-1];
                }else{
                    substring = rsd.getColumnClassName(i);
                }
                map.put(rsd.getColumnName(i),substring);
            }
            pst.close();
            pst=null;
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return map;
    }

    public List<AssetModel> getTableAssets(DataSource dataSource, String tableName){
        String jdbcUrl="";
        jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
        Connection conn = null;
        List<AssetModel> assetModels = new ArrayList<>();
        long staTime = new Date().getTime();
        Map<String, String> typeMap = getTypeMap(dataSource, tableName);
        long endTime = new Date().getTime();
        log.info("查询表字段类型信息耗时:"+(endTime-staTime)+"ms");
        try {
            conn = DriverManager.getConnection(jdbcUrl, dataSource.getUsername(), dataSource.getPassword());
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        try {
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet columnsRS = metaData.getColumns(dataSource.getDatabaseName(),dataSource.getModeName(), tableName, "%");
            ResultSet primaryKeyRS = metaData.getPrimaryKeys(dataSource.getDatabaseName(), dataSource.getModeName(), tableName);
            String primaryKeyColumnName ="";
            while (primaryKeyRS.next()) {
                primaryKeyColumnName = primaryKeyRS.getString("COLUMN_NAME");
            }
            while (columnsRS.next()) {
                AssetModel build = AssetModel.builder()
                        .createBy(SecurityUtils.getUsername())
                        .createTime(new Date())
                        .length(String.valueOf(columnsRS.getInt("COLUMN_SIZE")))
                        .decimalPlaces(String.valueOf(columnsRS.getInt("DECIMAL_DIGITS")))
                        .name(columnsRS.getString("COLUMN_NAME"))
                        .type(columnsRS.getString("TYPE_NAME"))
                        .mappingType(typeMap.get(columnsRS.getString("COLUMN_NAME")))
                        .dictKey("")
                        .isDict("")
                        .isNull(columnsRS.getString("IS_NULLABLE").substring(0, 1))
                        .isPrimaryKey(columnsRS.getString("COLUMN_NAME").equals(primaryKeyColumnName) ? "Y" : "N")
                        .comment(columnsRS.getString("REMARKS")==null || columnsRS.getString("REMARKS")=="" ? "-" : columnsRS.getString("REMARKS"))
                        .defaultValue(columnsRS.getString("COLUMN_DEF")==null || columnsRS.getString("COLUMN_DEF")=="" ? "-":columnsRS.getString("COLUMN_DEF"))
                        .build();
                assetModels.add(build);
            }
            columnsRS.close();
            primaryKeyRS.close();
            conn.close();
        } catch(SQLException e) {
            e.printStackTrace();
        }
        return assetModels;
    }

    public AssetsModule getAssets(DataSource dataSource){
        String jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
        Connection conn = null;
        HashMap<String, String> map = new HashMap<>();
        List<AssetModel> assetModels = getTableAssets(dataSource,dataSource.getTableName());
        try {
            Class.forName(dataSource.getJdbcDriver());
            conn = DriverManager.getConnection(jdbcUrl, dataSource.getUsername(), dataSource.getPassword());
        } catch (SQLException | ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        try {
            PreparedStatement pst = conn.prepareStatement("select * from "+dataSource.getTableName()+" where 1=1");
            ResultSet resultSet = pst.executeQuery();
            ResultSetMetaData rsd = resultSet.getMetaData();
            for(int i = 1; i <= rsd.getColumnCount(); i++) {
                String substring = rsd.getColumnClassName(i).substring(rsd.getColumnClassName(i).indexOf("java.lang.") + 10);
                map.put(rsd.getColumnName(i),substring);
            }
            for (AssetModel tableAsset : assetModels) {
                tableAsset.setMappingType(map.get(tableAsset.getType()));
            }
            pst.close();
        } catch(SQLException e) {
            e.printStackTrace();
        }
        AssetsModule assetsModule = new AssetsModule();
        assetsModule.setStructure(map);
        return assetsModule;
    }

    @Override
    public Result assetsList(DataSource dataSource) {
        AssetsModule kvt = getAssets(dataSource);
        return Result.success(kvt);
    }

    @Override
    @Transactional
    public Result structureList(DataAsset dataAsset) {
        AssetsModule kvt = getStructure(dataAsset);
        return Result.success(kvt);
    }

    @Override
    public Result synchronousData(DataSource dataSource) {
        deleteChildLevel(new Long[]{dataSource.getId()},"synchronous");
        String jdbcUrl = "";
        String sql;
        jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
        if (dataSource.getType().equals("MySql")){
            sql="SELECT TABLE_NAME t_name,TABLE_COMMENT table_comment,TABLE_ROWS table_rows,(SELECT count(*) FROM INFORMATION_SCHEMA.columns WHERE TABLE_SCHEMA = '"+dataSource.getDatabaseName()+"' and TABLE_NAME=t_name) fields FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA='"+dataSource.getDatabaseName()+"'";
        }else{
            sql="SELECT \n" +
                    "    c.relname AS t_name,\n" +
                    "    pgd.description AS table_comment,\n" +
                    "    COUNT(col.column_name) AS fields,\n" +
                    "    c.reltuples AS table_rows\n" +
                    "FROM \n" +
                    "    pg_catalog.pg_class c\n" +
                    "LEFT JOIN \n" +
                    "    pg_catalog.pg_namespace n ON n.oid = c.relnamespace\n" +
                    "LEFT JOIN \n" +
                    "    pg_catalog.pg_description pgd ON pgd.objoid = c.oid\n" +
                    "LEFT JOIN \n" +
                    "    information_schema.columns col ON c.relname = col.table_name AND n.nspname = col.table_schema\n" +
                    "WHERE \n" +
                    "    n.nspname = '"+dataSource.getModeName()+"' \n" +
                    "AND \n" +
                    "    c.relkind = 'r' \n" +
                    "GROUP BY \n" +
                    "    c.relname, pgd.description, c.reltuples";
        }
        Connection conn;
        HashMap<String, String> map = new HashMap<>();
        ArrayList<DataAsset> dataAssets = new ArrayList<>();
        try {
            long startTime = new Date().getTime();
            Class.forName(dataSource.getJdbcDriver());
            conn = DriverManager.getConnection(jdbcUrl, dataSource.getUsername(), dataSource.getPassword());

            Future<List<DataAsset>> dataAssetsFuture = executorService.submit(() -> {
                PreparedStatement ps = conn.prepareStatement(sql);
                ResultSet resultSet = ps.executeQuery();

                while (resultSet.next()) {
                    dataAssets.add(new DataAsset().dataAssetBuilder(dataSource, resultSet));
                }
                return dataAssets;
            });
            CompletableFuture<List<DataAsset>> completableFuture = CompletableFuture.completedFuture(null).thenCompose(unused -> {
                try {
                    return CompletableFuture.supplyAsync(() -> {
                        try {
                            return dataAssetsFuture.get(); // 等待Future完成并获取结果
                        } catch (InterruptedException | ExecutionException e) {
                            throw new RuntimeException("Error getting result from Future", e);
                        }
                    }, executorService);
                } catch (RuntimeException e) {
                    return CompletableFuture.failedFuture(e);
                }
            });
            List<AssetModel> assetModels = new ArrayList<>();
            completableFuture.thenAccept(dataAssetss -> {
                dataAssetMapper.batchInsert(dataAssets);
                List<CompletableFuture<Void>> assetModelFutures = new ArrayList<>();
                for (DataAsset dataAsset : dataAssets) {
                    CompletableFuture<Void> future = CompletableFuture.runAsync(() -> {
                        List<AssetModel> tableAssets = getTableAssets(dataSource, dataAsset.getTableName());
                        tableAssets.forEach(assetModel -> assetModel.setDataAssetId(dataAsset.getId()));
                        assetModels.addAll(tableAssets);
                    }, executorService);
                    assetModelFutures.add(future);
                }
                CompletableFuture.allOf(assetModelFutures.toArray(new CompletableFuture[0])).join();
                assetModelMapper.batchInsert(assetModels);
            });

            long endTime = new Date().getTime();
            log.info("处理总耗时: " + (endTime - startTime) + "ms");

            return Result.success();
        } catch (SQLException | ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Result dataAssetList(DataSource dataSource) {
        DataAsset dataAsset = new DataAsset();
        dataAsset.setDataSourceId(dataSource.getId());
        List<DataAsset> dataAssetList = new ArrayList<>();
        List<DataAsset> dataAssets = dataAssetMapper.selectDataAssetList(dataAsset);
        List<SysRole> roles = SecurityUtils.getLoginUser().getSysUser().getRoles();
        if (roles.get(0).getRoleId()==1){
            dataAssetList = dataAssets;
        }else{
            List<Long> assetIds = assetAccreditMapper.getAssetAccreditByUserId(SecurityUtils.getUserId()).stream().map(AssetAccredit::getDataAssetId).filter(Objects::nonNull).toList();
            dataAssetList = dataAssets.stream().filter(dataAssetInfo -> assetIds.contains(dataAssetInfo.getId())).toList();
        }
        return Result.success(dataAssetList);
    }

    @Override
    public Result assetModelList(DataAsset dataAsset) {
        AssetModel assetModel = new AssetModel();
        assetModel.setDataAssetId(dataAsset.getId());
        List<AssetModel> assetModels = assetModelMapper.selectAssetModelList(assetModel);
        List<String> strings = assetModels.stream().map(AssetModel::getDictKey).toList();
        if (!strings.isEmpty()){
            Result<List<Dictionary>> result = dictionaryService.getDictionaryDataList(strings);
            List<Dictionary> data = result.getData();
            assetModels.stream()
                    .forEach(assetModelInfo -> {
                        data.stream().forEach(dataInfo -> {
                            if (assetModelInfo.getDictKey().equals(dataInfo.getDictionaryKey())){
                                assetModelInfo.setDictionaryDatas(dataInfo.getDictionaryDataList());
                            }
                        });
                    });
        }
        return Result.success(assetModels);
    }

    @Override
    public Result statistics() {
        //查询所有数据源
        List<DataSource> dataSourceList = new ArrayList<>();
        List<DataSource> dataSources = dataSourceMapper.selectDataSourceList(new DataSource());
        List<SysRole> roles = SecurityUtils.getLoginUser().getSysUser().getRoles();
        List<Long> assetIds;
        if (roles.get(0).getRoleId()==1){
            assetIds = new ArrayList<>();
            dataSourceList = dataSources;
        }else{
            ArrayList<Long> longs1 = new ArrayList<>();
            List<Long> sourceIds = sourceAccreditMapper.getSourceAccreditByUserId(SecurityUtils.getUserId()).stream().map(SourceAccredit::getDataSourceId).filter(Objects::nonNull).toList();
            assetIds = assetAccreditMapper.getAssetAccreditByUserId(SecurityUtils.getUserId()).stream().map(AssetAccredit::getDataAssetId).filter(Objects::nonNull).toList();
            if (!assetIds.isEmpty()){
                List<Long> longs = dataAssetMapper.getDataAssetByAssetId(assetIds).stream().map(DataAsset::getDataSourceId).toList();
                longs1.addAll(longs);
            }
            longs1.addAll(sourceIds);
            dataSourceList = dataSources.stream().filter(dataSourceInfo -> longs1.contains(dataSourceInfo.getId())).toList();
        }
        //获取所有数据源的主键
        List<Long> longs = dataSourceList.stream().map(dataSource -> dataSource.getId()).toList();
        Statistics statistics = new Statistics();
        if (!longs.isEmpty()){
            List<DataAsset> dataAssetList=dataAssetMapper.selectDataAssetBatchId(longs);
            if (roles.get(0).getRoleId()!=1){
                dataAssetList = dataAssetList.stream().filter(dataAsset -> assetIds.contains(dataAsset.getId())).toList();
            }

            statistics.setDataAsset(Long.valueOf(dataSourceList.size()));
            long sum1;
            long sum2=0;
            sum1 = dataAssetList.stream().mapToLong(dataAsset -> Long.valueOf(dataAsset.getFields())).sum();
            sum2 = dataAssetList.stream().mapToLong(dataAsset -> Long.valueOf(dataAsset.getTableCount())).sum();
            statistics.setAssetModel(sum1);
            statistics.setDataModel(sum2);
        }else{
            statistics.setAssetModel(0L);
            statistics.setDataModel(0L);
        }

        return Result.success(statistics);
    }
}
