package com.muyu.etl.service.impl;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.SysUser;
import com.muyu.common.system.remote.RemoteUserService;
import com.muyu.etl.domain.SourceAccredit;
import com.muyu.etl.domain.model.AccreditModel;
import com.muyu.etl.domain.req.SourceAccreditReq;
import com.muyu.etl.mapper.SourceAccreditMapper;
import com.muyu.etl.service.ISourceAccreditService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-25
 */
@Service
public class SourceAccreditServiceImpl implements ISourceAccreditService
{
    @Autowired
    private SourceAccreditMapper sourceAccreditMapper;

    @Autowired
    private RemoteUserService remoteUserService;

    /**
     * 查询【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public SourceAccredit selectSourceAccreditById(Long id)
    {
        return sourceAccreditMapper.selectSourceAccreditById(id);
    }

    /**
     * 查询【请填写功能名称】列表
     *
     * @param sourceAccredit 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<SourceAccredit> selectSourceAccreditList(SourceAccredit sourceAccredit)
    {
        return sourceAccreditMapper.selectSourceAccreditList(sourceAccredit);
    }

    /**
     * 新增【请填写功能名称】
     *
     * @param sourceAccreditReq 【请填写功能名称】
     * @return 结果
     */
    @Override
    @Transactional
    public Result insertSourceAccredit(SourceAccreditReq sourceAccreditReq)
    {
        deleteSourceAccreditBySourceIds(sourceAccreditReq);
        List<SourceAccredit> sourceAccredits;
        if (sourceAccreditReq.getUserId() != null){
            sourceAccredits = new ArrayList<>();
            SourceAccredit sourceAccredit = new SourceAccredit();
            sourceAccredit.setDataSourceId(sourceAccreditReq.getDataSourceId());
            sourceAccredit.setUserId(sourceAccreditReq.getUserId());
            sourceAccredit.setCreateBy(SecurityUtils.getUsername());
            sourceAccredit.setCreateTime(new Date());
            sourceAccredits.add(sourceAccredit);
        }else{

            sourceAccredits = new ArrayList<>(sourceAccreditReq.getDeptIds().stream().map(deptId -> {
                SourceAccredit sourceAccredit = new SourceAccredit();
                sourceAccredit.setDataSourceId(sourceAccreditReq.getDataSourceId());
                sourceAccredit.setDeptId(deptId);
                sourceAccredit.setCreateBy(SecurityUtils.getUsername());
                sourceAccredit.setCreateTime(new Date());
                return sourceAccredit;
            }).toList());
            List<Long> deptIds = sourceAccreditReq.getDeptIds();
            Result<List<SysUser>> list = remoteUserService.userList(new SysUser());
            List<SysUser> sysUsers = list.getData();
            sourceAccredits.addAll(sysUsers.stream().filter(sysUser -> deptIds.contains(sysUser.getDeptId())).toList().stream().map(sysUser -> {
                SourceAccredit sourceAccredit = new SourceAccredit();
                sourceAccredit.setDataSourceId(sourceAccreditReq.getDataSourceId());
                sourceAccredit.setUserId(sysUser.getUserId());
                sourceAccredit.setCreateBy(SecurityUtils.getUsername());
                sourceAccredit.setCreateTime(new Date());
                return sourceAccredit;
            }).toList());
        }
        sourceAccreditMapper.insertBatchSourceAccredit(sourceAccredits);
        return Result.success();
    }

    /**
     * 修改【请填写功能名称】
     *
     * @param sourceAccredit 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int updateSourceAccredit(SourceAccredit sourceAccredit)
    {
        sourceAccredit.setUpdateTime(DateUtils.getNowDate());
        return sourceAccreditMapper.updateSourceAccredit(sourceAccredit);
    }

    /**
     * 批量删除【请填写功能名称】
     *
     * @param ids 需要删除的【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteSourceAccreditByIds(Long[] ids)
    {
        return sourceAccreditMapper.deleteSourceAccreditByIds(ids);
    }

    /**
     * 删除【请填写功能名称】信息
     *
     * @param id 【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteSourceAccreditById(Long id)
    {
        return sourceAccreditMapper.deleteSourceAccreditById(id);
    }

    @Override
    public Result getSourceAccreditByDataSourceId(Long id) {
        List<SourceAccredit> sourceAccreditList = sourceAccreditMapper.getSourceAccreditByDataSourceId(id);
        List<Long> userAccreditIds = sourceAccreditList.stream().map(SourceAccredit::getUserId).filter(Objects::nonNull).toList();
        List<Long> deptAccreditIds = sourceAccreditList.stream().map(SourceAccredit::getDeptId).filter(Objects::nonNull).toList();
        AccreditModel accreditModel = new AccreditModel();
        accreditModel.setDeptAccreditModelIds(deptAccreditIds);
        accreditModel.setUserAccreditModelIds(userAccreditIds);
        return Result.success(accreditModel);
    }

    @Override
    public Result deleteSourceAccreditBySourceIds(SourceAccreditReq sourceAccreditReq) {
        if (sourceAccreditReq.getUserId() != null){
            sourceAccreditMapper.deleteSourceAccreditByDeptUser(null,sourceAccreditReq.getUserId(),sourceAccreditReq.getDataSourceId());
        }else{
            List<Long> deptIds = sourceAccreditReq.getDeptIds();
            Result<List<SysUser>> list = remoteUserService.userList(new SysUser());
            List<SysUser> sysUsers = list.getData();
            List<Long> userIds = sysUsers.stream()
                    .filter(sysUser -> deptIds.contains(sysUser.getDeptId())).toList().stream()
                        .filter(sysUser -> deptIds.contains(sysUser.getDeptId())).toList().stream()
                            .map(SysUser::getUserId).toList();
            sourceAccreditMapper.deleteSourceAccreditByUser(userIds,sourceAccreditReq.getDataSourceId());
            sourceAccreditMapper.deleteSourceAccreditByDeptUser(sourceAccreditReq.getDeptIds(),null,sourceAccreditReq.getDataSourceId());
        }
        return Result.success();
    }
}
