package com.muyu.rule.engine.client.config;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.rule.engine.RemoteRuleEngineService;
import com.muyu.rule.engine.RemoteRuleVersionService;
import com.muyu.rule.engine.constant.GenerateConstant;
import com.muyu.rule.engine.constant.PathConstant;
import com.muyu.rule.engine.domain.EngineMaintenance;
import com.muyu.rule.engine.domain.EngineVersion;
import com.muyu.rule.engine.initialize.MyRuleEngine;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import javax.tools.JavaCompiler;
import javax.tools.JavaFileObject;
import javax.tools.StandardJavaFileManager;
import javax.tools.ToolProvider;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * @ClassName RuleEngineClientRunner
 * @Description 描述
 * @Author Xin.Yao
 * @Date 2024/5/9 14:42
 */
@Component
@Log4j2
public class RuleEngineClientRunner implements ApplicationRunner {
    @Autowired
    private RemoteRuleVersionService remoteRuleVersionService;
    @Override
    public void run(ApplicationArguments args) throws Exception {
        Result<TableDataInfo<EngineVersion>> list = remoteRuleVersionService.list(new EngineVersion());
        List<EngineVersion> engineVersionList = list.getData().getRows();
        engineVersionList.stream().forEach(engineVersion -> {
            try {
                String className = GenerateConstant.getClassName(engineVersion.getVersionCode());
                String fileName = PathConstant.JAVA_PATH+className+".java";
                File file = new File(fileName);
                File fileParent = file.getParentFile();
                if(!fileParent.exists()){
                    fileParent.mkdir();
                }
                if (file.exists()){
                    file.delete();
                }
                file.createNewFile();
                FileWriter fileWriter = new FileWriter(file);
                fileWriter.write(engineVersion.getCodeIng());
                fileWriter.flush();
                fileWriter.close();
                JavaCompiler jc = ToolProvider.getSystemJavaCompiler();
                StandardJavaFileManager fileManager = jc.getStandardFileManager(null, null, null);
                Iterable fileObjects = fileManager.getJavaFileObjects(fileName);
                List<String> options = Arrays.asList("-d", PathConstant.CLASS_PATH);
                JavaCompiler.CompilationTask cTask = jc.getTask(null, fileManager, null, options, null, fileObjects);
                Boolean call = cTask.call();
                fileManager.close();
                if (call){
                    String cName = PathConstant.PACKAGE_PATH_SERVICE+GenerateConstant.getClassName(engineVersion.getVersionCode());
                    String classPath = PathConstant.CLASS_PATH+cName.replace(".", "\\")+".class";
                    MyRuleEngine.setRuleVersionClassPath(engineVersion.getVersionCode()+"_"+engineVersion.getId(),classPath);
                }
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        });
    }
}
















