package com.muyu.rule.engine.constant;

import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.rule.engine.domain.EngineMaintenance;
import com.muyu.rule.engine.domain.EngineVersion;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @ClassName GenerateConstant
 * @Description 描述
 * @Author Xin.Yao
 * @Date 2024/5/7 19:45
 */
@Component
public class GenerateConstant {
    public static final String DATA_FIELD = "DataModelEngine";
    public static final String DATA_RECORD = "RecordEngine";
    public static final String DATA_SET = "DataSetEngine";
    public static final String ACTION_IMPORT = "import com.muyu.rule.engine.action.ActionDiscard;\n";
    public static final String SCOPE_IMPORT = "import com.muyu.rule.engine.scope.";
    public static final String PACKAGE_PATH = "package com.muyu.rule.engine.custom;";

    public static String getClassName(String versionCode){
        String[] splits = versionCode.split("_");
        String className="";
        for (String split : splits) {
            className += split.substring(0,1).toUpperCase()+split.substring(1);
        }
        return className;
    }

    public static String generateConstant(EngineMaintenance engineMaintenance, EngineVersion engineVersion){
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        String format = simpleDateFormat.format(new Date());
        String level = selectType(engineMaintenance.getLevel());
        return  PACKAGE_PATH+"\n\n" +
                ACTION_IMPORT +
                SCOPE_IMPORT+level+";\n" +
                "\n" +
                "/**\n" +
                " * @Author: "+ SecurityUtils.getUsername() +"\n" +
                " * @date: "+format+"\n" +
                " * @Description: "+engineVersion.getName()+engineVersion.getCode()+"\n" +
                " * @Version: 1.0\n" +
                " */\n" +
                "public class "+getClassName(engineVersion.getVersionCode())+" extends "+level+" {\n" +
                "    @Override\n" +
                "    public void execution () {\n" +
                "    }\n" +
                "}";
    }

    public static String selectType(String level){
        String type = "";
        switch (level) {
            case "data-field":
                type = DATA_FIELD;
                break;
            case "data-record":
                type = DATA_RECORD;
                break;
            case "data-set":
                type = DATA_SET;
                break;
        }
        return type;
    }
}
