package com.muyu.rule.engine.consumer;

import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson2.JSON;
import com.muyu.etl.datasource.MyDataSource;
import com.muyu.etl.domain.DataSource;
import com.muyu.rule.engine.config.RabbitAdminConfig;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.sql.SQLException;

/**
 * @ClassName DataSourceConsumer
 * @Description 描述
 * @Author Xin.Yao
 * @Date 2024/5/16 20:37
 */
@Component
@Log4j2
public class DataSourceConsumer {
    @RabbitListener(queuesToDeclare = {@Queue(name = RabbitAdminConfig.DATA_SOURCE_EXCHANGE)})
    public void cusConfirmConfig(String msg, Message message, Channel channel){
        try {
            DataSource dataSource = JSON.parseObject(msg, DataSource.class);
            log.info(RabbitAdminConfig.DATA_SOURCE_EXCHANGE+"接收到的消息,消息内容为:{}",dataSource.toString());
            String jdbcUrl = "jdbc:"+dataSource.getType().toLowerCase()+"://"+dataSource.getLinkAddress()+":"+dataSource.getPort()+"/"+dataSource.getDatabaseName();
            String key = dataSource.getDataSourceName()+"_"+dataSource.getId();
            DruidDataSource druidDataSource = new DruidDataSource();
            //和数据源建立连接
            druidDataSource.setUrl(jdbcUrl);
            druidDataSource.setUsername(dataSource.getUsername());
            druidDataSource.setPassword(dataSource.getPassword());
            //配置连接池信息
            druidDataSource.setInitialSize(Integer.valueOf(dataSource.getInitNum().toString()));
            druidDataSource.setMaxActive(Integer.valueOf(dataSource.getMaxNum().toString()));
            druidDataSource.setMinIdle(5);
            druidDataSource.init();
            MyDataSource.addDataSource(key,druidDataSource);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
            log.info("消息消费成功...");
        } catch (IOException e) {
            log.error("消费消息出现了异常,异常信息:{}",e.getMessage());
            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),true);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        } catch (SQLException e) {
            log.error("创建连接池出现问题");
            throw new RuntimeException(e);
        }

    }
}
