package com.muyu.rule.engine.service.impl;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Method;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;

import com.muyu.common.core.constant.TokenConstants;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.rule.engine.ClassLoading.CustomClassLoader;
import com.muyu.rule.engine.constant.PathConstant;
import com.muyu.rule.engine.domain.EngineVersion;
import com.muyu.rule.engine.mapper.EngineVersionMapper;
import com.muyu.rule.engine.req.DataSourceAssetModelReq;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.rule.engine.mapper.EngineMaintenanceMapper;
import com.muyu.rule.engine.domain.EngineMaintenance;
import com.muyu.rule.engine.service.IEngineMaintenanceService;

import javax.tools.JavaCompiler;
import javax.tools.StandardJavaFileManager;
import javax.tools.ToolProvider;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author ruoyi
 * @date 2024-05-02
 */
@Service
@Log4j2
public class EngineMaintenanceServiceImpl implements IEngineMaintenanceService
{
    @Autowired
    private EngineMaintenanceMapper engineMaintenanceMapper;

    @Autowired
    private EngineVersionMapper engineVersionMapper;

    /**
     * 查询【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public EngineMaintenance selectEngineMaintenanceById(Long id)
    {
        EngineVersion engineVersion = new EngineVersion();
        engineVersion.setEngineMaintenanceId(id);
        // 根据维护引擎ID查询所有关联的版本信息
        List<EngineVersion> engineVersions = engineVersionMapper.selectEngineVersionList(engineVersion);
        // 根据维护引擎ID查询对应的维护信息
        EngineMaintenance engineMaintenance = engineMaintenanceMapper.selectEngineMaintenanceById(id);
        // 将查询到的版本信息列表设置到维护信息对象中
        engineMaintenance.setEngineVersionList(engineVersions);

        return engineMaintenance;
    }

    /**
     * 查询【请填写功能名称】列表
     *
     * @param engineMaintenance 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<EngineMaintenance> selectEngineMaintenanceList(EngineMaintenance engineMaintenance)
    {
        return engineMaintenanceMapper.selectEngineMaintenanceList(engineMaintenance);
    }

    /**
     * 新增【请填写功能名称】
     *
     * @param engineMaintenance 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int insertEngineMaintenance(EngineMaintenance engineMaintenance)
    {
        String className = "Rule"+Character.toUpperCase(engineMaintenance.getCode().charAt(0)) + engineMaintenance.getCode().substring(1)+"Class";
        engineMaintenance.setCreateTime(DateUtils.getNowDate());
        engineMaintenance.setCreateBy(SecurityUtils.getUsername());
        // 构造完整的类代码并设置到engineMaintenance对象中
        engineMaintenance.setCodeText("package com.muyu.rule.engine.domain;\n\n\n"+
                "public class " + className + " {\n" +
                "}");
        // 设置引擎的自定义代码
        engineMaintenance.setEngineCode("engine_custom_"+engineMaintenance.getCode());
        // 插入引擎维护信息到数据库
        return engineMaintenanceMapper.insertEngineMaintenance(engineMaintenance);

    }

    /**
     * 更新引擎维护信息
     *
     * @param engineMaintenance 引擎维护对象,包含需要更新的信息
     * @return 更新成功与否的标志
     */
    @Override
    public int updateEngineMaintenance(EngineMaintenance engineMaintenance)
    {
        engineMaintenance.setUpdateTime(DateUtils.getNowDate()); // 设置更新时间为当前时间
        engineMaintenance.setUpdateBy(SecurityUtils.getUsername()); // 设置更新操作人的用户名
        return engineMaintenanceMapper.updateEngineMaintenance(engineMaintenance); // 调用mapper接口更新引擎维护信息

    }

    /**
     * 批量删除【请填写功能名称】
     *
     * @param ids 需要删除的【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteEngineMaintenanceByIds(Long[] ids)
    {
        return engineMaintenanceMapper.deleteEngineMaintenanceByIds(ids);
    }

    /**
     * 删除【请填写功能名称】信息
     *
     * @param id 【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteEngineMaintenanceById(Long id)
    {
        return engineMaintenanceMapper.deleteEngineMaintenanceById(id);
    }

    @Override
    public Result testMethod(String code) {
        String className = "com.muyu.rule.engine.domain.Rule"+Character.toUpperCase(code.charAt(0)) + code.substring(1)+"Class";
        String classPath = "D:\\work\\ruoyi-cloud-server\\muyu-modules\\muyu-rule-engine\\target\\classes\\"+className.replace(".", "\\")+".class";
        try {
            // 读取类文件
            byte[] classData = Files.readAllBytes(Paths.get(classPath));
            CustomClassLoader customClassLoader = new CustomClassLoader();
            Class<?> aClass = customClassLoader.defineClassFromBytes(className,classData);
            // 实例化类
            Object obj = aClass.newInstance();
            //获取类中所有方法
            Method[] methods = aClass.getDeclaredMethods();
            for (Method method : methods) {
                method.invoke(obj);
            }
            customClassLoader = null;
        } catch (Exception e) {
            log.error(e.getMessage());
            Result.error("测试失败");
        }

        return Result.success("测试成功");
    }

    public String getTypeCodeText(EngineMaintenance engineMaintenance) {
        String className =Character.toUpperCase(engineMaintenance.getCode().charAt(0)) + engineMaintenance.getCode().substring(1);
        switch (engineMaintenance.getType()) {
            case "data-set":
                className = className+"DataSetContext";
                return "package com.muyu.rule.engine.domain;\n" +
                        "\n" +
                        "public class "+className+" {\n" +
                        "\n" +
                        "    private final RecordContext recordContext;\n" +
                        "\n" +
                        "    public DataSetContext (RecordContext recordContext) {\n" +
                        "        this.recordContext = recordContext;\n" +
                        "    }\n" +
                        "}\n";
            case "data-record":
                return "记录";
            default:
                return "数据字段";
        }
    }
}
