package com.muyu.rule.engine.service.impl;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Method;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.sql.*;
import java.util.*;

import com.alibaba.fastjson.JSON;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.etl.datasource.MyDataSource;
import com.muyu.etl.domain.custom.VTClass;
import com.muyu.rule.engine.ClassLoading.CustomClassLoader;
import com.muyu.rule.engine.constant.GenerateConstant;
import com.muyu.rule.engine.constant.PathConstant;
import com.muyu.rule.engine.domain.EngineMaintenance;
import com.muyu.rule.engine.mapper.EngineMaintenanceMapper;
import com.muyu.rule.engine.req.DataSourceAssetModelReq;
import com.muyu.rule.engine.req.TestInterfaceReq;
import com.muyu.rule.scope.DataModelContext;
import com.muyu.rule.scope.DataSetContext;
import com.muyu.rule.scope.RecordContext;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.rule.engine.mapper.EngineVersionMapper;
import com.muyu.rule.engine.domain.EngineVersion;
import com.muyu.rule.engine.service.IEngineVersionService;

import javax.tools.JavaCompiler;
import javax.tools.StandardJavaFileManager;
import javax.tools.ToolProvider;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author ruoyi
 * @date 2024-05-06
 */
@Service
@Log4j2
public class EngineVersionServiceImpl implements IEngineVersionService
{
    @Autowired
    private EngineVersionMapper engineVersionMapper;

    @Autowired
    private EngineMaintenanceMapper engineMaintenanceMapper;

    /**
     * 查询【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public EngineVersion selectEngineVersionById(Long id)
    {
        return engineVersionMapper.selectEngineVersionById(id);
    }

    /**
     * 查询【请填写功能名称】列表
     *
     * @param engineVersion 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<EngineVersion> selectEngineVersionList(EngineVersion engineVersion)
    {
        return engineVersionMapper.selectEngineVersionList(engineVersion);
    }

    /**
     * 新增【请填写功能名称】
     *
     * @param engineVersion 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int insertEngineVersion(EngineVersion engineVersion)
    {
        engineVersion.setCreateTime(DateUtils.getNowDate());
        engineVersion.setCreateBy(SecurityUtils.getUsername());
        return engineVersionMapper.insertEngineVersion(engineVersion);
    }

    /**
     * 修改【请填写功能名称】
     *
     * @param engineVersion 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int updateEngineVersion(EngineVersion engineVersion)
    {
        engineVersion.setUpdateTime(DateUtils.getNowDate());
        engineVersion.setUpdateBy(SecurityUtils.getUsername());
        int i = engineVersionMapper.updateEngineVersion(engineVersion);
        EngineVersion engineVersion1 = engineVersionMapper.selectEngineVersionById(engineVersion.getId());
        initializeRuleEngine(engineVersion1);
        return i;
    }

    /**
     * 批量删除【请填写功能名称】
     *
     * @param ids 需要删除的【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteEngineVersionByIds(Long[] ids)
    {
        return engineVersionMapper.deleteEngineVersionByIds(ids);
    }

    /**
     * 删除【请填写功能名称】信息
     *
     * @param id 【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteEngineVersionById(Long id)
    {
        return engineVersionMapper.deleteEngineVersionById(id);
    }

    @Override
    public Result generatedCode(EngineVersion engineVersion) {
        //根据引擎ID获取规则引擎信息
        EngineMaintenance engineMaintenance = engineMaintenanceMapper.selectEngineMaintenanceById(engineVersion.getEngineMaintenanceId());
        //生成规则版本类名
        engineVersion.setVersionCode(engineMaintenance.getEngineCode()+"_"+engineVersion.getCode());
        //生成规则版本代码
        engineVersion.setCodeIng(GenerateConstant.generateConstant(engineMaintenance,engineVersion));
        return Result.success(engineVersion);
    }

    @Override
    public Result initializeRuleEngine(EngineVersion engineVersion) {
        try {
            //创建源文件
            String className = GenerateConstant.getClassName(engineVersion.getVersionCode());
            // 源文件路径和名称
            String filename = PathConstant.JAVA_PATH +className+".java";
            File file = new File(filename);
            // 确保源文件所在的目录存在
            File fileParent = file.getParentFile();

            if (!fileParent.exists()) {
                fileParent.mkdir();
            }
            // 确保源文件存在，如果已存在则先删除再创建
            if (file.exists()) {
                file.delete(); // 删除存在的文件
            }
            file.createNewFile();
            // 将源代码写入文件
            FileWriter fw = new FileWriter(file);
            fw.write(engineVersion.getCodeIng());
            fw.flush();
            fw.close();
            // 使用JavaCompiler 编译java文件
            // 获取系统Java编译器
            JavaCompiler jc = ToolProvider.getSystemJavaCompiler();
            // 获取标准文件管理器
            StandardJavaFileManager fileManager = jc.getStandardFileManager(null, null, null);
            // 获取要编译的文件对象
            Iterable fileObjects = fileManager.getJavaFileObjects(filename);
            // 设置编译选项，指定输出目录
            List<String> options = Arrays.asList("-d", PathConstant.CLASS_PATH);
            // 创建编译任务
            JavaCompiler.CompilationTask cTask   = jc.getTask(null, fileManager, null, options, null, fileObjects);
            // 执行编译任务
            Boolean call = cTask.call();
            // 关闭文件管理器
            fileManager.close();
            if (call){
                insertEngineVersion(engineVersion);
                return Result.success("保存成功");
            }
        }catch (Exception e){
            log.error(e.getMessage());
            Result.error("初始化失败");
        }
        return Result.error("初始化失败");
    }

    @Override
    public Result getDataRegionList(DataSourceAssetModelReq dataSourceAssetModelReq) {
        ArrayList<Map<String, VTClass>> kvtList = new ArrayList<>();
        log.info("数据源: "+dataSourceAssetModelReq.getDataSource().toString());
        try {
            String sql = "select * from "+(dataSourceAssetModelReq.getDataSource().getModeName() == null ? "" : dataSourceAssetModelReq.getDataSource().getModeName()+".")+dataSourceAssetModelReq.getDataAsset().getTableName() +" limit 5";
            Connection connection = MyDataSource.getConnection(dataSourceAssetModelReq.getDataSource().getDataSourceName() + "_" + dataSourceAssetModelReq.getDataSource().getId());
            PreparedStatement pst = connection.prepareStatement(sql);
            ResultSet resultSet = pst.executeQuery();
            ResultSetMetaData rsd = resultSet.getMetaData();

            int columnCount = rsd.getColumnCount();
            // 遍历查询结果，将每行数据转换为 Map 形式存储
            while (resultSet.next()){
                Map<String, VTClass> stringVTClassHashMap = new HashMap<>();
                for (int i = 1; i <= columnCount; i++) {
                    // 获取列名和类型，并封装数据
                    String columnName = rsd.getColumnName(i);
                    String[] split = rsd.getColumnClassName(i).split("\\.");
                    String type = split[split.length-1];
                    Object value = resultSet.getObject(i);
                    if (value==null){
                        stringVTClassHashMap.put(columnName,new VTClass("",type));
                    }else{
                        stringVTClassHashMap.put(columnName,new VTClass(value.toString(),type));
                    }
                }
                kvtList.add(stringVTClassHashMap);
            }
            // 关闭资源
            pst.close();
            MyDataSource.addBack(connection);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return Result.success(kvtList);
    }

    @Override
    public Result testRuleVersion(TestInterfaceReq testInterfaceReq) {
        EngineMaintenance engineMaintenance = engineMaintenanceMapper.selectEngineMaintenanceById(testInterfaceReq.getEngineVersion().getEngineMaintenanceId());
        if (engineMaintenance.getLevel().equals("data-feild")){
            DataModelContext.set(testInterfaceReq.getDataProcessModel());
        }else if(engineMaintenance.getLevel().equals("data-set")){
            DataSetContext.set(testInterfaceReq.getDataSetProcessModel());
        }else{
            RecordContext.set(testInterfaceReq.getRecordProcessModel());
        }
        String className = PathConstant.PACKAGE_PATH_SERVICE+GenerateConstant.getClassName(testInterfaceReq.getEngineVersion().getVersionCode());
        String classPath = PathConstant.CLASS_PATH+className.replace(".", "\\")+".class";
        try {
            // 读取类文件
            byte[] classData = Files.readAllBytes(Paths.get(classPath));
            CustomClassLoader customClassLoader = new CustomClassLoader();
            Class<?> aClass = customClassLoader.defineClassFromBytes(className,classData);
            // 实例化类
            Object obj = aClass.newInstance();
            //获取类中所有方法
            Method[] methods = aClass.getDeclaredMethods();
            for (Method method : methods) {
                method.invoke(obj);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            return Result.success("程序异常: 信息["+e.getLocalizedMessage()+"]异常");
        }
        return Result.success("测试正常,无异常数据返回");
    }
}
